"""test_resources"""
import unittest
from nose.tools import eq_, assert_raises

from routes import *

class TestResourceGeneration(unittest.TestCase):
    def _assert_restful_routes(self, m, options, path_prefix=''):
        baseroute = '/' + path_prefix + options['controller']
        eq_(baseroute, m.generate(action='index', **options))
        eq_(baseroute + '.xml', m.generate(action='index', format='xml', **options))
        eq_(baseroute + '/new', m.generate(action='new', **options))
        eq_(baseroute + '/1', m.generate(action='show', id='1', **options))
        eq_(baseroute + '/1/edit', m.generate(action='edit',id='1', **options))
        eq_(baseroute + '/1.xml', m.generate(action='show', id='1',format='xml', **options))

        eq_(baseroute, m.generate(action='create', method='post', **options))
        eq_(baseroute + '/1', m.generate(action='update', method='put', id='1', **options))
        eq_(baseroute + '/1', m.generate(action='delete', method='delete', id='1', **options))

    def test_resources(self):
        m = Mapper()
        m.resource('message', 'messages')
        m.resource('massage', 'massages')
        m.resource('passage', 'passages')
        m.create_regs(['messages'])
        options = dict(controller='messages')
        eq_('/messages', url_for('messages'))
        eq_('/messages.xml', url_for('formatted_messages', format='xml'))
        eq_('/messages/1', url_for('message', id=1))
        eq_('/messages/1.xml', url_for('formatted_message', id=1, format='xml'))
        eq_('/messages/new', url_for('new_message'))
        eq_('/messages/1.xml', url_for('formatted_message', id=1, format='xml'))
        eq_('/messages/1/edit', url_for('edit_message', id=1))
        eq_('/messages/1/edit.xml', url_for('formatted_edit_message', id=1, format='xml'))
        self._assert_restful_routes(m, options)

    def test_resources_with_path_prefix(self):
        m = Mapper()
        m.resource('message', 'messages', path_prefix='/thread/:threadid')
        m.create_regs(['messages'])
        options = dict(controller='messages', threadid='5')
        self._assert_restful_routes(m, options, path_prefix='thread/5/')

    def test_resources_with_collection_action(self):
        m = Mapper()
        m.resource('message', 'messages', collection=dict(rss='GET'))
        m.create_regs(['messages'])
        options = dict(controller='messages')
        self._assert_restful_routes(m, options)
        eq_('/messages/rss', m.generate(controller='messages', action='rss'))
        eq_('/messages/rss', url_for('rss_messages'))
        eq_('/messages/rss.xml', m.generate(controller='messages', action='rss', format='xml'))
        eq_('/messages/rss.xml', url_for('formatted_rss_messages', format='xml'))

    def test_resources_with_member_action(self):
        for method in ['put', 'post']:
            m = Mapper()
            m.resource('message', 'messages', member=dict(mark=method))
            m.create_regs(['messages'])
            options = dict(controller='messages')
            self._assert_restful_routes(m, options)
            eq_('/messages/1/mark', m.generate(method=method, action='mark', id='1', **options))
            eq_('/messages/1/mark.xml',
                m.generate(method=method, action='mark', id='1', format='xml', **options))

    def test_resources_with_new_action(self):
        m = Mapper()
        m.resource('message', 'messages/', new=dict(preview='POST'))
        m.create_regs(['messages'])
        options = dict(controller='messages')
        self._assert_restful_routes(m, options)
        eq_('/messages/new/preview', m.generate(controller='messages', action='preview', method='post'))
        eq_('/messages/new/preview', url_for('preview_new_message'))
        eq_('/messages/new/preview.xml',
            m.generate(controller='messages', action='preview', method='post', format='xml'))
        eq_('/messages/new/preview.xml', url_for('formatted_preview_new_message', format='xml'))

    def test_resources_with_name_prefix(self):
        m = Mapper()
        m.resource('message', 'messages', name_prefix='category_', new=dict(preview='POST'))
        m.create_regs(['messages'])
        options = dict(controller='messages')
        self._assert_restful_routes(m, options)
        eq_('/messages/new/preview', url_for('category_preview_new_message'))
        assert_raises(Exception, url_for, 'category_preview_new_message', method='get')

    def test_resources_with_requirements(self):
        m = Mapper()
        m.resource('message', 'messages', path_prefix='/{project_id}/{user_id}/',
                   requirements={'project_id': r'[0-9a-f]{4}', 'user_id': r'\d+'})
        options = dict(controller='messages', project_id='cafe', user_id='123')
        self._assert_restful_routes(m, options, path_prefix='cafe/123/')

        # in addition to the positive tests we need to guarantee we
        # are not matching when the requirements don't match.
        eq_({'action': u'create', 'project_id': u'cafe', 'user_id': u'123', 'controller': u'messages'},
            m.match('/cafe/123/messages'))
        eq_(None, m.match('/extensions/123/messages'))
        eq_(None, m.match('/b0a3/123b/messages'))
        eq_(None, m.match('/foo/bar/messages'))


class TestResourceRecognition(unittest.TestCase):
    def test_resource(self):
        m = Mapper()
        m.resource('person', 'people')
        m.create_regs(['people'])

        con = request_config()
        con.mapper = m
        def test_path(path, method):
            env = dict(HTTP_HOST='example.com', PATH_INFO=path, REQUEST_METHOD=method)
            con.mapper_dict = {}
            con.environ = env

        test_path('/people', 'GET')
        eq_({'controller':'people', 'action':'index'}, con.mapper_dict)
        test_path('/people.xml', 'GET')
        eq_({'controller':'people', 'action':'index', 'format':'xml'}, con.mapper_dict)

        test_path('/people', 'POST')
        eq_({'controller':'people', 'action':'create'}, con.mapper_dict)
        test_path('/people.html', 'POST')
        eq_({'controller':'people', 'action':'create', 'format':'html'}, con.mapper_dict)

        test_path('/people/2.xml', 'GET')
        eq_({'controller':'people', 'action':'show', 'id':'2', 'format':'xml'}, con.mapper_dict)
        test_path('/people/2', 'GET')
        eq_({'controller':'people', 'action':'show', 'id':'2'}, con.mapper_dict)

        test_path('/people/2/edit', 'GET')
        eq_({'controller':'people', 'action':'edit', 'id':'2'}, con.mapper_dict)
        test_path('/people/2/edit.xml', 'GET')
        eq_({'controller':'people', 'action':'edit', 'id':'2', 'format':'xml'}, con.mapper_dict)

        test_path('/people/2', 'DELETE')
        eq_({'controller':'people', 'action':'delete', 'id':'2'}, con.mapper_dict)

        test_path('/people/2', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'2'}, con.mapper_dict        )
        test_path('/people/2.json', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'2', 'format':'json'}, con.mapper_dict        )

        # Test for dots in urls
        test_path('/people/2\.13', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'2\.13'}, con.mapper_dict)
        test_path('/people/2\.13.xml', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'2\.13', 'format':'xml'}, con.mapper_dict)
        test_path('/people/user\.name', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'user\.name'}, con.mapper_dict)
        test_path('/people/user\.\.\.name', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'user\.\.\.name'}, con.mapper_dict)
        test_path('/people/user\.name\.has\.dots', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'user\.name\.has\.dots'}, con.mapper_dict)
        test_path('/people/user\.name\.is\.something.xml', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'user\.name\.is\.something', 'format':'xml'}, con.mapper_dict)
        test_path('/people/user\.name\.ends\.with\.dot\..xml', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'user\.name\.ends\.with\.dot\.', 'format':'xml'}, con.mapper_dict)
        test_path('/people/user\.name\.ends\.with\.dot\.', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'user\.name\.ends\.with\.dot\.'}, con.mapper_dict)
        test_path('/people/\.user\.name\.starts\.with\.dot', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'\.user\.name\.starts\.with\.dot'}, con.mapper_dict)
        test_path('/people/user\.name.json', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'user\.name', 'format':'json'}, con.mapper_dict)

    def test_resource_with_nomin(self):
        m = Mapper()
        m.minimization = False
        m.resource('person', 'people')
        m.create_regs(['people'])

        con = request_config()
        con.mapper = m
        def test_path(path, method):
            env = dict(HTTP_HOST='example.com', PATH_INFO=path, REQUEST_METHOD=method)
            con.mapper_dict = {}
            con.environ = env

        test_path('/people', 'GET')
        eq_({'controller':'people', 'action':'index'}, con.mapper_dict)

        test_path('/people', 'POST')
        eq_({'controller':'people', 'action':'create'}, con.mapper_dict)

        test_path('/people/2', 'GET')
        eq_({'controller':'people', 'action':'show', 'id':'2'}, con.mapper_dict)
        test_path('/people/2/edit', 'GET')
        eq_({'controller':'people', 'action':'edit', 'id':'2'}, con.mapper_dict)

        test_path('/people/2', 'DELETE')
        eq_({'controller':'people', 'action':'delete', 'id':'2'}, con.mapper_dict)

        test_path('/people/2', 'PUT')
        eq_({'controller':'people', 'action':'update', 'id':'2'}, con.mapper_dict)

    def test_resource_created_with_parent_resource(self):
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'))
        m.create_regs(['locations'])

        con = request_config()
        con.mapper = m
        def test_path(path, method):
            env = dict(HTTP_HOST='example.com', PATH_INFO=path,
                       REQUEST_METHOD=method)
            con.mapper_dict = {}
            con.environ = env

        test_path('/regions/13/locations', 'GET')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'action': 'index'})
        url = url_for('region_locations', region_id=13)
        eq_(url, '/regions/13/locations')

        test_path('/regions/13/locations', 'POST')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'action': 'create'})
        # new
        url = url_for('region_new_location', region_id=13)
        eq_(url, '/regions/13/locations/new')
        # create
        url = url_for('region_locations', region_id=13)
        eq_(url, '/regions/13/locations')

        test_path('/regions/13/locations/60', 'GET')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'id': '60', 'action': 'show'})
        url = url_for('region_location', region_id=13, id=60)
        eq_(url, '/regions/13/locations/60')

        test_path('/regions/13/locations/60/edit', 'GET')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'id': '60', 'action': 'edit'})
        url = url_for('region_edit_location', region_id=13, id=60)
        eq_(url, '/regions/13/locations/60/edit')

        test_path('/regions/13/locations/60', 'DELETE')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'id': '60', 'action': 'delete'})
        url = url_for('region_location', region_id=13, id=60)
        eq_(url, '/regions/13/locations/60')

        test_path('/regions/13/locations/60', 'PUT')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'id': '60', 'action': 'update'})
        url = url_for('region_location', region_id=13, id=60)
        eq_(url, '/regions/13/locations/60')

        # Make sure ``path_prefix`` overrides work
        # empty ``path_prefix`` (though I'm not sure why someone would do this)
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='')
        url = url_for('region_locations')
        eq_(url, '/locations')
        # different ``path_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='areas/:area_id')
        url = url_for('region_locations', area_id=51)
        eq_(url, '/areas/51/locations')

        # Make sure ``name_prefix`` overrides work
        # empty ``name_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   name_prefix='')
        url = url_for('locations', region_id=51)
        eq_(url, '/regions/51/locations')
        # different ``name_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   name_prefix='area_')
        url = url_for('area_locations', region_id=51)
        eq_(url, '/regions/51/locations')

        # Make sure ``path_prefix`` and ``name_prefix`` overrides work together
        # empty ``path_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='',
                   name_prefix='place_')
        url = url_for('place_locations')
        eq_(url, '/locations')
        # empty ``name_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='areas/:area_id',
                   name_prefix='')
        url = url_for('locations', area_id=51)
        eq_(url, '/areas/51/locations')
        # different ``path_prefix`` and ``name_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='areas/:area_id',
                   name_prefix='place_')
        url = url_for('place_locations', area_id=51)
        eq_(url, '/areas/51/locations')

    def test_resource_created_with_parent_resource_nomin(self):
        m = Mapper()
        m.minimization = False
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'))
        m.create_regs(['locations'])

        con = request_config()
        con.mapper = m
        def test_path(path, method):
            env = dict(HTTP_HOST='example.com', PATH_INFO=path,
                       REQUEST_METHOD=method)
            con.mapper_dict = {}
            con.environ = env

        test_path('/regions/13/locations', 'GET')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'action': 'index'})
        url = url_for('region_locations', region_id=13)
        eq_(url, '/regions/13/locations')

        test_path('/regions/13/locations', 'POST')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'action': 'create'})
        # new
        url = url_for('region_new_location', region_id=13)
        eq_(url, '/regions/13/locations/new')
        # create
        url = url_for('region_locations', region_id=13)
        eq_(url, '/regions/13/locations')

        test_path('/regions/13/locations/60', 'GET')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'id': '60', 'action': 'show'})
        url = url_for('region_location', region_id=13, id=60)
        eq_(url, '/regions/13/locations/60')

        test_path('/regions/13/locations/60/edit', 'GET')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'id': '60', 'action': 'edit'})
        url = url_for('region_edit_location', region_id=13, id=60)
        eq_(url, '/regions/13/locations/60/edit')

        test_path('/regions/13/locations/60', 'DELETE')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'id': '60', 'action': 'delete'})
        url = url_for('region_location', region_id=13, id=60)
        eq_(url, '/regions/13/locations/60')

        test_path('/regions/13/locations/60', 'PUT')
        eq_(con.mapper_dict, {'region_id': '13', 'controller': 'locations',
                                   'id': '60', 'action': 'update'})
        url = url_for('region_location', region_id=13, id=60)
        eq_(url, '/regions/13/locations/60')

        # Make sure ``path_prefix`` overrides work
        # empty ``path_prefix`` (though I'm not sure why someone would do this)
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='/')
        url = url_for('region_locations')
        eq_(url, '/locations')
        # different ``path_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='areas/:area_id')
        url = url_for('region_locations', area_id=51)
        eq_(url, '/areas/51/locations')

        # Make sure ``name_prefix`` overrides work
        # empty ``name_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   name_prefix='')
        url = url_for('locations', region_id=51)
        eq_(url, '/regions/51/locations')
        # different ``name_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   name_prefix='area_')
        url = url_for('area_locations', region_id=51)
        eq_(url, '/regions/51/locations')

        # Make sure ``path_prefix`` and ``name_prefix`` overrides work together
        # empty ``path_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='',
                   name_prefix='place_')
        url = url_for('place_locations')
        eq_(url, '/locations')
        # empty ``name_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='areas/:area_id',
                   name_prefix='')
        url = url_for('locations', area_id=51)
        eq_(url, '/areas/51/locations')
        # different ``path_prefix`` and ``name_prefix``
        m = Mapper()
        m.resource('location', 'locations',
                   parent_resource=dict(member_name='region',
                                        collection_name='regions'),
                   path_prefix='areas/:area_id',
                   name_prefix='place_')
        url = url_for('place_locations', area_id=51)
        eq_(url, '/areas/51/locations')



if __name__ == '__main__':
    unittest.main()



import unittest
from routes import Mapper

class TestMapperStr(unittest.TestCase):
    def test_str(self):
        m = Mapper()
        m.connect('/{controller}/{action}')
        m.connect('entries', '/entries', controller='entry', action='index')
        m.connect('entry', '/entries/{id}', controller='entry', action='show')

        expected = """\
Route name Methods Path                   Controller action
                   /{controller}/{action}
entries            /entries               entry      index
entry              /entries/{id}          entry      show"""

        for expected_line, actual_line in zip(expected.splitlines(), str(m).splitlines()):
            assert expected_line == actual_line.rstrip()



from routes import Mapper
from routes.middleware import RoutesMiddleware
from webtest import TestApp
from nose.tools import eq_

def simple_app(environ, start_response):
    route_dict = environ['wsgiorg.routing_args'][1]
    start_response('200 OK', [('Content-type', 'text/plain')])
    items = list(route_dict.items())
    items.sort()
    return [('The matchdict items are %s and environ is %s' % (items, environ)).encode()]

def test_basic():
    map = Mapper(explicit=False)
    map.minimization = True
    map.connect(':controller/:action/:id')
    map.create_regs(['content'])
    app = TestApp(RoutesMiddleware(simple_app, map))
    res = app.get('/')
    assert b'matchdict items are []' in res

    res = app.get('/content')
    assert b"matchdict items are [('action', 'index'), ('controller', " + repr(
        u'content').encode() + b"), ('id', None)]" in res

def test_no_query():
    map = Mapper(explicit=False)
    map.minimization = True
    map.connect('myapp/*path_info', controller='myapp')
    map.connect('project/*path_info', controller='myapp')
    map.create_regs(['content', 'myapp'])

    app = RoutesMiddleware(simple_app, map)
    env = {'PATH_INFO': '/', 'REQUEST_METHOD': 'GET', 'HTTP_HOST': 'localhost'}
    def start_response_wrapper(status, headers, exc=None):
        pass
    response = b''.join(app(env, start_response_wrapper))
    assert b'matchdict items are []' in response

def test_content_split():
    map = Mapper(explicit=False)
    map.minimization = True
    map.connect('myapp/*path_info', controller='myapp')
    map.connect('project/*path_info', controller='myapp')
    map.create_regs(['content', 'myapp'])

    app = RoutesMiddleware(simple_app, map)
    env = {'PATH_INFO': '/', 'REQUEST_METHOD': 'POST', 'CONTENT_TYPE': 'text/plain;text/html',
           'HTTP_HOST': 'localhost'}
    def start_response_wrapper(status, headers, exc=None):
        pass
    response = b''.join(app(env, start_response_wrapper))
    assert b'matchdict items are []' in response

def test_no_singleton():
    map = Mapper(explicit=False)
    map.minimization = True
    map.connect('myapp/*path_info', controller='myapp')
    map.connect('project/*path_info', controller='myapp')
    map.create_regs(['content', 'myapp'])

    app = RoutesMiddleware(simple_app, map, singleton=False)
    env = {'PATH_INFO': '/', 'REQUEST_METHOD': 'POST', 'CONTENT_TYPE': 'text/plain;text/html'}
    def start_response_wrapper(status, headers, exc=None):
        pass
    response = b''.join(app(env, start_response_wrapper))
    assert b'matchdict items are []' in response

    # Now a match
    env = {'PATH_INFO': '/project/fred', 'REQUEST_METHOD': 'POST', 'CONTENT_TYPE': 'text/plain;text/html'}
    def start_response_wrapper(status, headers, exc=None):
        pass
    response = b''.join(app(env, start_response_wrapper))
    assert b"matchdict items are [('action', " + repr(u'index').encode() + \
           b"), ('controller', " + repr(u'myapp').encode() + b"), ('path_info', 'fred')]" in response


def test_path_info():
    map = Mapper(explicit=False)
    map.minimization = True
    map.connect('myapp/*path_info', controller='myapp')
    map.connect('project/*path_info', controller='myapp')
    map.create_regs(['content', 'myapp'])

    app = TestApp(RoutesMiddleware(simple_app, map))
    res = app.get('/')
    assert 'matchdict items are []' in res

    res = app.get('/myapp/some/other/url')
    print(res)
    assert b"matchdict items are [('action', " + repr(u'index').encode() + \
           b"), ('controller', " + repr(u'myapp').encode() + b"), ('path_info', 'some/other/url')]" in res
    assert "'SCRIPT_NAME': '/myapp'" in res
    assert "'PATH_INFO': '/some/other/url'" in res

    res = app.get('/project/pylonshq/browser/pylons/templates/default_project/+package+/pylonshq/browser/pylons/templates/default_project/+package+/controllers')
    print(res)
    assert "'SCRIPT_NAME': '/project'" in res
    assert "'PATH_INFO': '/pylonshq/browser/pylons/templates/default_project/+package+/pylonshq/browser/pylons/templates/default_project/+package+/controllers'" in res

def test_redirect_middleware():
    map = Mapper(explicit=False)
    map.minimization = True
    map.connect('myapp/*path_info', controller='myapp')
    map.redirect("faq/{section}", "/static/faq/{section}.html")
    map.redirect("home/index", "/", _redirect_code='301 Moved Permanently')
    map.create_regs(['content', 'myapp'])

    app = TestApp(RoutesMiddleware(simple_app, map))
    res = app.get('/')
    assert 'matchdict items are []' in res

    res = app.get('/faq/home')
    eq_('302 Found', res.status)
    eq_(res.headers['Location'], '/static/faq/home.html')

    res = app.get('/myapp/some/other/url')
    print(res)
    assert b"matchdict items are [('action', " + repr(u'index').encode() + \
           b"), ('controller', " + repr(u'myapp').encode() + \
           b"), ('path_info', 'some/other/url')]" in res
    assert "'SCRIPT_NAME': '/myapp'" in res
    assert "'PATH_INFO': '/some/other/url'" in res

    res = app.get('/home/index')
    assert '301 Moved Permanently' in res.status
    eq_(res.headers['Location'], '/')

def test_method_conversion():
    map = Mapper(explicit=False)
    map.minimization = True
    map.connect('content/:type', conditions=dict(method='DELETE'))
    map.connect(':controller/:action/:id')
    map.create_regs(['content'])
    app = TestApp(RoutesMiddleware(simple_app, map))
    res = app.get('/')
    assert 'matchdict items are []' in res

    res = app.get('/content')
    assert b"matchdict items are [('action', 'index'), ('controller', " + \
           repr(u'content').encode() + b"), ('id', None)]" in res

    res = app.get('/content/hopper', params={'_method':'DELETE'})
    assert b"matchdict items are [('action', " + repr(u'index').encode() + \
           b"), ('controller', " + repr(u'content').encode() + \
           b"), ('type', " + repr(u'hopper').encode() + b")]" in res

    res = app.post('/content/grind',
                   params={'_method':'DELETE', 'name':'smoth'},
                   headers={'Content-Type': 'application/x-www-form-urlencoded'})
    assert b"matchdict items are [('action', " + repr(u'index').encode() + \
           b"), ('controller', " + repr(u'content').encode() + \
           b"), ('type', " + repr(u'grind').encode() + b")]" in res
    assert "'REQUEST_METHOD': 'POST'" in res

    #res = app.post('/content/grind',
    #               upload_files=[('fileupload', 'hello.txt', 'Hello World')],
    #               params={'_method':'DELETE', 'name':'smoth'})
    #assert "matchdict items are [('action', u'index'), ('controller', u'content'), ('type', u'grind')]" in res
    #assert "'REQUEST_METHOD': 'POST'" in res



try:
    import profile
    import pstats
except ImportError:
    pass
import tempfile
import os
import time
from routes import Mapper

def get_mapper():
    m = Mapper()
    m.connect('', controller='articles', action='index')
    m.connect('admin', controller='admin/general', action='index')

    m.connect('admin/comments/article/:article_id/:action/:id',
              controller = 'admin/comments', action = None, id=None)
    m.connect('admin/trackback/article/:article_id/:action/:id',
              controller='admin/trackback', action=None, id=None)
    m.connect('admin/content/:action/:id', controller='admin/content')

    m.connect('xml/:action/feed.xml', controller='xml')
    m.connect('xml/articlerss/:id/feed.xml', controller='xml',
              action='articlerss')
    m.connect('index.rdf', controller='xml', action='rss')

    m.connect('articles', controller='articles', action='index')
    m.connect('articles/page/:page', controller='articles',
              action='index', requirements = {'page':'\d+'})

    m.connect(
        'articles/:year/:month/:day/page/:page',
        controller='articles', action='find_by_date', month = None,
        day = None,
        requirements = {'year':'\d{4}', 'month':'\d{1,2}','day':'\d{1,2}'})

    m.connect('articles/category/:id', controller='articles', action='category')
    m.connect('pages/*name', controller='articles', action='view_page')
    m.create_regs(['content','admin/why', 'admin/user'])
    return m

def bench_rec(mapper, n):
    ts = time.time()
    for x in range(1,n):
        pass
    en = time.time()

    match = mapper.match

    # hits
    start = time.time()
    for x in range(1,n):
        match('/admin')
        match('/xml/1/feed.xml')
        match('/index.rdf')
    end = time.time()
    total = end-start-(en-ts)
    per_url = total / (n*10)
    print("Hit recognition\n")
    print("%s ms/url" % (per_url*1000))
    print("%s urls/s\n" % (1.00/per_url))

    # misses
    start = time.time()
    for x in range(1,n):
        match('/content')
        match('/content/list')
        match('/content/show/10')
    end = time.time()
    total = end-start-(en-ts)
    per_url = total / (n*10)
    print("Miss recognition\n")
    print("%s ms/url" % (per_url*1000))
    print("%s urls/s\n" % (1.00/per_url))
        
def do_profile(cmd, globals, locals, sort_order, callers):
    fd, fn = tempfile.mkstemp()
    try:
        if hasattr(profile, 'runctx'):
            profile.runctx(cmd, globals, locals, fn)
        else:
            raise NotImplementedError(
                'No profiling support under Python 2.3')
        stats = pstats.Stats(fn)
        stats.strip_dirs()
        # calls,time,cumulative and cumulative,calls,time are useful
        stats.sort_stats(*sort_order or ('cumulative', 'calls', 'time'))
        if callers:
            stats.print_callers()
        else:
            stats.print_stats()
    finally:
        os.remove(fn)

def main(n=300):
    mapper = get_mapper()
    do_profile('bench_rec(mapper, %s)' % n, globals(), locals(),
               ('time', 'cumulative', 'calls'), None)

if __name__ == '__main__':
    main()
    



"""test_explicit_use"""
import os, sys, time, unittest
from nose.tools import eq_, assert_raises, assert_is_none

from routes import *
from routes.route import Route
from routes.util import GenerationException

class TestUtils(unittest.TestCase):
    def test_route_dict_use(self):
        m = Mapper()
        m.explicit = True
        m.connect('/hi/{fred}')

        environ = {'HTTP_HOST': 'localhost'}

        env = environ.copy()
        env['PATH_INFO'] = '/hi/george'

        eq_({'fred': 'george'}, m.match(environ=env))

    def test_x_forwarded(self):
        m = Mapper()
        m.explicit = True
        m.connect('/hi/{fred}')

        environ = {'HTTP_X_FORWARDED_HOST': 'localhost'}
        url = URLGenerator(m, environ)
        eq_('http://localhost/hi/smith', url(fred='smith', qualified=True))

    def test_server_port(self):
        m = Mapper()
        m.explicit = True
        m.connect('/hi/{fred}')

        environ = {'SERVER_NAME': 'localhost', 'wsgi.url_scheme': 'https',
                   'SERVER_PORT': '993'}
        url = URLGenerator(m, environ)
        eq_('https://localhost:993/hi/smith', url(fred='smith', qualified=True))

    def test_subdomain_screen(self):
        m = Mapper()
        m.explicit = True
        m.sub_domains = True
        m.connect('/hi/{fred}')

        environ = {'HTTP_HOST': 'localhost.com'}
        url = URLGenerator(m, environ)
        eq_('http://home.localhost.com/hi/smith', url(fred='smith', sub_domain=u'home', qualified=True))

        environ = {'HTTP_HOST': 'here.localhost.com', 'PATH_INFO': '/hi/smith'}
        url = URLGenerator(m, environ.copy())
        assert_raises(GenerationException, lambda: url.current(qualified=True))

        environ = {'HTTP_HOST': 'subdomain.localhost.com'}
        url = URLGenerator(m, environ.copy())
        eq_('http://sub.localhost.com/hi/smith', url(fred='smith', sub_domain='sub', qualified=True))

        environ = {'HTTP_HOST': 'sub.sub.localhost.com'}
        url = URLGenerator(m, environ.copy())
        eq_('http://new.localhost.com/hi/smith', url(fred='smith', sub_domain='new', qualified=True))

        url = URLGenerator(m, {})
        eq_('/hi/smith', url(fred='smith', sub_domain=u'home'))

    def test_anchor(self):
        m = Mapper()
        m.explicit = True
        m.connect('/hi/{fred}')

        environ = {'HTTP_HOST': 'localhost.com'}
        url = URLGenerator(m, environ)
        eq_('/hi/smith#here', url(fred='smith', anchor='here'))

    def test_static_args(self):
        m = Mapper()
        m.explicit = True
        m.connect('http://google.com/', _static=True)

        url = URLGenerator(m, {})

        eq_('/here?q=fred&q=here%20now', url('/here', q=[u'fred', 'here now']))

    def test_current(self):
        m = Mapper()
        m.explicit = True
        m.connect('/hi/{fred}')

        environ = {'HTTP_HOST': 'localhost.com', 'PATH_INFO': '/hi/smith'}
        match = m.routematch(environ=environ)[0]
        environ['wsgiorg.routing_args'] = (None, match)
        url = URLGenerator(m, environ)
        eq_('/hi/smith', url.current())

    def test_add_routes(self):
        map = Mapper(explicit=True)
        map.minimization = False
        routes = [
            Route('foo', '/foo',)
        ]
        map.extend(routes)
        eq_(map.match('/foo'), {})

    def test_add_routes_conditions_unmet(self):
        map = Mapper(explicit=True)
        map.minimization = False
        routes = [
            Route('foo', '/foo', conditions=dict(method=["POST"]))
        ]
        environ = {
            'HTTP_HOST': 'localhost.com',
            'PATH_INFO': '/foo',
            'REQUEST_METHOD': 'GET',
        }
        map.extend(routes)
        assert_is_none(map.match('/foo', environ=environ))

    def test_add_routes_conditions_met(self):
        map = Mapper(explicit=True)
        map.minimization = False
        routes = [
            Route('foo', '/foo', conditions=dict(method=["POST"]))
        ]
        environ = {
            'HTTP_HOST': 'localhost.com',
            'PATH_INFO': '/foo',
            'REQUEST_METHOD': 'POST',
        }
        map.extend(routes)
        eq_(map.match('/foo', environ=environ), {})

    def test_using_func(self):
        def fred(view):
            pass

        m = Mapper()
        m.explicit = True
        m.connect('/hi/{fred}', controller=fred)

        environ = {'HTTP_HOST': 'localhost.com', 'PATH_INFO': '/hi/smith'}
        match = m.routematch(environ=environ)[0]
        environ['wsgiorg.routing_args'] = (None, match)
        url = URLGenerator(m, environ)
        eq_('/hi/smith', url.current())

    def test_using_prefix(self):
        m = Mapper()
        m.explicit = True
        m.connect('/{first}/{last}')

        environ = {'HTTP_HOST': 'localhost.com', 'PATH_INFO': '/content/index',
                   'SCRIPT_NAME': '/jones'}
        match = m.routematch(environ=environ)[0]
        environ['wsgiorg.routing_args'] = (None, match)
        url = URLGenerator(m, environ)

        eq_('/jones/content/index', url.current())
        eq_('/jones/smith/barney', url(first='smith', last='barney'))

    def test_with_host_param(self):
        m = Mapper()
        m.explicit = True
        m.connect('/hi/{fred}')

        environ = {'HTTP_HOST': 'localhost.com'}
        url = URLGenerator(m, environ)
        eq_('/hi/smith?host=here', url(fred='smith', host_='here'))



"""Test non-minimization recognition"""
from six.moves import urllib

from nose.tools import eq_

from routes import url_for
from routes.mapper import Mapper


def test_basic():
    m = Mapper(explicit=False)
    m.minimization = False
    m.connect('/:controller/:action/:id')
    m.create_regs(['content'])

    # Recognize
    eq_(None, m.match('/content'))
    eq_(None, m.match('/content/index'))
    eq_(None, m.match('/content/index/'))
    eq_({'controller':'content','action':'index','id':'4'},
        m.match('/content/index/4'))
    eq_({'controller':'content','action':'view','id':'4.html'},
        m.match('/content/view/4.html'))

    # Generate
    eq_(None, m.generate(controller='content'))
    eq_('/content/index/4', m.generate(controller='content', id=4))
    eq_('/content/view/3', m.generate(controller='content', action='view', id=3))

def test_full():
    m = Mapper(explicit=False)
    m.minimization = False
    m.connect('/:controller/:action/', id=None)
    m.connect('/:controller/:action/:id')
    m.create_regs(['content'])

    # Recognize
    eq_(None, m.match('/content'))
    eq_(None, m.match('/content/index'))
    eq_({'controller':'content','action':'index','id':None},
        m.match('/content/index/'))
    eq_({'controller':'content','action':'index','id':'4'},
        m.match('/content/index/4'))
    eq_({'controller':'content','action':'view','id':'4.html'},
        m.match('/content/view/4.html'))

    # Generate
    eq_(None, m.generate(controller='content'))

    # Looks odd, but only controller/action are set with non-explicit, so we
    # do need the id to match
    eq_('/content/index/', m.generate(controller='content', id=None))
    eq_('/content/index/4', m.generate(controller='content', id=4))
    eq_('/content/view/3', m.generate(controller='content', action='view', id=3))

def test_action_required():
    m = Mapper()
    m.minimization = False
    m.explicit = True
    m.connect('/:controller/index', action='index')
    m.create_regs(['content'])

    eq_(None, m.generate(controller='content'))
    eq_(None, m.generate(controller='content', action='fred'))
    eq_('/content/index', m.generate(controller='content', action='index'))

def test_query_params():
    m = Mapper()
    m.minimization = False
    m.explicit = True
    m.connect('/:controller/index', action='index')
    m.create_regs(['content'])

    eq_(None, m.generate(controller='content'))
    eq_('/content/index?test=sample',
        m.generate(controller='content', action='index', test='sample'))


def test_syntax():
    m = Mapper(explicit=False)
    m.minimization = False
    m.connect('/{controller}/{action}/{id}')
    m.create_regs(['content'])

    # Recognize
    eq_(None, m.match('/content'))
    eq_(None, m.match('/content/index'))
    eq_(None, m.match('/content/index/'))
    eq_({'controller':'content','action':'index','id':'4'},
        m.match('/content/index/4'))

    # Generate
    eq_(None, m.generate(controller='content'))
    eq_('/content/index/4', m.generate(controller='content', id=4))
    eq_('/content/view/3', m.generate(controller='content', action='view', id=3))

def test_regexp_syntax():
    m = Mapper(explicit=False)
    m.minimization = False
    m.connect('/{controller}/{action}/{id:\d\d}')
    m.create_regs(['content'])

    # Recognize
    eq_(None, m.match('/content'))
    eq_(None, m.match('/content/index'))
    eq_(None, m.match('/content/index/'))
    eq_(None, m.match('/content/index/3'))
    eq_({'controller':'content','action':'index','id':'44'},
        m.match('/content/index/44'))

    # Generate
    eq_(None, m.generate(controller='content'))
    eq_(None, m.generate(controller='content', id=4))
    eq_('/content/index/43', m.generate(controller='content', id=43))
    eq_('/content/view/31', m.generate(controller='content', action='view', id=31))

def test_unicode():
    hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
    hoge_enc = urllib.parse.quote(hoge.encode('utf-8'))
    m = Mapper()
    m.minimization = False
    m.connect(':hoge')
    eq_("/%s" % hoge_enc, m.generate(hoge=hoge))
    assert isinstance(m.generate(hoge=hoge), str)

def test_unicode_static():
    hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
    hoge_enc = urllib.parse.quote(hoge.encode('utf-8'))
    m = Mapper()
    m.minimization = False
    m.connect('google-jp', 'http://www.google.co.jp/search', _static=True)
    m.create_regs(['messages'])
    eq_("http://www.google.co.jp/search?q=" + hoge_enc,
                     url_for('google-jp', q=hoge))
    assert isinstance(url_for('google-jp', q=hoge), str)

def test_other_special_chars():
    m = Mapper()
    m.minimization = False
    m.connect('/:year/:(slug).:(format),:(locale)', locale='en', format='html')
    m.create_regs(['content'])

    eq_('/2007/test.xml,ja', m.generate(year=2007, slug='test', format='xml', locale='ja'))
    eq_(None, m.generate(year=2007, format='html'))



"""test_resources"""
import unittest
from nose.tools import eq_, assert_raises

from routes import *

class TestSubmapper(unittest.TestCase):
    def test_submapper(self):
        m = Mapper()
        c = m.submapper(path_prefix='/entries', requirements=dict(id='\d+'))
        c.connect('entry', '/{id}')

        eq_('/entries/1', url_for('entry', id=1))
        assert_raises(Exception, url_for, 'entry', id='foo')

    def test_submapper_with_no_path(self):
        m = Mapper()
        c = m.submapper(path_prefix='/')
        c.connect('entry')
        eq_('/entry?id=1', url_for('entry', id=1))

    def test_submapper_nesting(self):
        m = Mapper()
        c = m.submapper(path_prefix='/entries', controller='entry',
                        requirements=dict(id='\d+'))
        e = c.submapper(path_prefix='/{id}')

        eq_('entry', c.resource_name)
        eq_('entry', e.resource_name)

        e.connect('entry', '')
        e.connect('edit_entry', '/edit')

        eq_('/entries/1', url_for('entry', id=1))
        eq_('/entries/1/edit', url_for('edit_entry', id=1))
        assert_raises(Exception, url_for, 'entry', id='foo')

    def test_submapper_action(self):
        m = Mapper(explicit=True)
        c = m.submapper(path_prefix='/entries', controller='entry')

        c.action(name='entries', action='list')
        c.action(action='create', method='POST')

        eq_('/entries', url_for('entries', method='GET'))
        eq_('/entries', url_for('create_entry', method='POST'))
        eq_('/entries', url_for(controller='entry', action='list', method='GET'))
        eq_('/entries', url_for(controller='entry', action='create', method='POST'))
        assert_raises(Exception, url_for, 'entries', method='DELETE')

    def test_submapper_link(self):
        m = Mapper(explicit=True)
        c = m.submapper(path_prefix='/entries', controller='entry')

        c.link(rel='new')
        c.link(rel='ping', method='POST')

        eq_('/entries/new', url_for('new_entry', method='GET'))
        eq_('/entries/ping', url_for('ping_entry', method='POST'))
        eq_('/entries/new', url_for(controller='entry', action='new', method='GET'))
        eq_('/entries/ping', url_for(controller='entry', action='ping', method='POST'))
        assert_raises(Exception, url_for, 'new_entry', method='PUT')
        assert_raises(Exception, url_for, 'ping_entry', method='PUT')

    def test_submapper_standard_actions(self):
        m = Mapper()
        c = m.submapper(path_prefix='/entries', collection_name='entries',
                        controller='entry')
        e = c.submapper(path_prefix='/{id}')

        c.index()
        c.create()
        e.show()
        e.update()
        e.delete()

        eq_('/entries', url_for('entries', method='GET'))
        eq_('/entries', url_for('create_entry', method='POST'))
        assert_raises(Exception, url_for, 'entries', method='DELETE')

        eq_('/entries/1', url_for('entry', id=1, method='GET'))
        eq_('/entries/1', url_for('update_entry', id=1, method='PUT'))
        eq_('/entries/1', url_for('delete_entry', id=1, method='DELETE'))
        assert_raises(Exception, url_for, 'entry', id=1, method='POST')

    def test_submapper_standard_links(self):
        m = Mapper()
        c = m.submapper(path_prefix='/entries', controller='entry')
        e = c.submapper(path_prefix='/{id}')

        c.new()
        e.edit()

        eq_('/entries/new', url_for('new_entry', method='GET'))
        assert_raises(Exception, url_for, 'new_entry', method='POST')

        eq_('/entries/1/edit', url_for('edit_entry', id=1, method='GET'))
        assert_raises(Exception, url_for, 'edit_entry', id=1, method='POST')

    def test_submapper_action_and_link_generation(self):
        m = Mapper()
        c = m.submapper(path_prefix='/entries', controller='entry',
                        collection_name='entries',
                        actions=['index', 'new', 'create'])
        e = c.submapper(path_prefix='/{id}',
                       actions=['show', 'edit', 'update', 'delete'])

        eq_('/entries', url_for('entries', method='GET'))
        eq_('/entries', url_for('create_entry', method='POST'))
        assert_raises(Exception, url_for, 'entries', method='DELETE')

        eq_('/entries/1', url_for('entry', id=1, method='GET'))
        eq_('/entries/1', url_for('update_entry', id=1, method='PUT'))
        eq_('/entries/1', url_for('delete_entry', id=1, method='DELETE'))
        assert_raises(Exception, url_for, 'entry', id=1, method='POST')

        eq_('/entries/new', url_for('new_entry', method='GET'))
        assert_raises(Exception, url_for, 'new_entry', method='POST')

        eq_('/entries/1/edit', url_for('edit_entry', id=1, method='GET'))
        assert_raises(Exception, url_for, 'edit_entry', id=1, method='POST')

    def test_collection(self):
        m = Mapper()
        c = m.collection('entries', 'entry')

        eq_('/entries', url_for('entries', method='GET'))
        eq_('/entries', url_for('create_entry', method='POST'))
        assert_raises(Exception, url_for, 'entries', method='DELETE')

        eq_('/entries/1', url_for('entry', id=1, method='GET'))
        eq_('/entries/1', url_for('update_entry', id=1, method='PUT'))
        eq_('/entries/1', url_for('delete_entry', id=1, method='DELETE'))
        assert_raises(Exception, url_for, 'entry', id=1, method='POST')

        eq_('/entries/new', url_for('new_entry', method='GET'))
        assert_raises(Exception, url_for, 'new_entry', method='POST')

        eq_('/entries/1/edit', url_for('edit_entry', id=1, method='GET'))
        assert_raises(Exception, url_for, 'edit_entry', id=1, method='POST')

    def test_collection_options(self):
        m = Mapper()
        requirement=dict(id='\d+')
        c = m.collection('entries', 'entry', conditions=dict(sub_domain=True),
                         requirements=requirement)
        for r in m.matchlist:
            eq_(True, r.conditions['sub_domain'])
            eq_(requirement, r.reqs)

    def test_subsubmapper_with_controller(self):
        m = Mapper()
        col1 = m.collection('parents', 'parent',
                            controller='col1',
                            member_prefix='/{parent_id}')
        # NOTE: If one uses functions as controllers, the error will be here.
        col2 = col1.member.collection('children', 'child',
                                      controller='col2',
                                      member_prefix='/{child_id}')
        match = m.match('/parents/1/children/2')
        eq_('col2', match.get('controller'))

    def test_submapper_argument_overriding(self):
        m = Mapper()
        first = m.submapper(path_prefix='/first_level',
                            controller='first', action='test',
                            name_prefix='first_')
        first.connect('test', r'/test')
        second = first.submapper(path_prefix='/second_level',
                                 controller='second',
                                 name_prefix='second_')
        second.connect('test', r'/test')
        third = second.submapper(path_prefix='/third_level',
                                 controller="third", action='third_action',
                                 name_prefix='third_')
        third.connect('test', r'/test')

        # test first level
        match = m.match('/first_level/test')
        eq_('first', match.get('controller'))
        eq_('test', match.get('action'))
        # test name_prefix worked
        eq_('/first_level/test', url_for('first_test'))

        # test second level controller override
        match = m.match('/first_level/second_level/test')
        eq_('second', match.get('controller'))
        eq_('test', match.get('action'))
        # test name_prefix worked
        eq_('/first_level/second_level/test', url_for('first_second_test'))

        # test third level controller and action override
        match = m.match('/first_level/second_level/third_level/test')
        eq_('third', match.get('controller'))
        eq_('third_action', match.get('action'))
        # test name_prefix worked
        eq_('/first_level/second_level/third_level/test',
             url_for('first_second_third_test'))


if __name__ == '__main__':
    unittest.main()



__version__ = '2.5.0'

import io
import os
import sys

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with io.open(os.path.join(here, 'README.rst'), encoding='utf8') as f:
    README = f.read()
with io.open(os.path.join(here, 'CHANGELOG.rst'), encoding='utf8') as f:
    CHANGES = f.read()
PY3 = sys.version_info[0] == 3

extra_options = {
    "packages": find_packages(),
    }

extras_require = {
    'middleware': [
        'webob',
    ]
}
extras_require['docs'] = ['Sphinx'] + extras_require['middleware']

if PY3:
    if "test" in sys.argv or "develop" in sys.argv:
        for root, directories, files in os.walk("tests"):
            for directory in directories:
                extra_options["packages"].append(os.path.join(root, directory))

setup(name="Routes",
      version=__version__,
      description='Routing Recognition and Generation Tools',
      long_description=README + '\n\n' + CHANGES,
      classifiers=["Development Status :: 5 - Production/Stable",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: MIT License",
                   "Topic :: Internet :: WWW/HTTP",
                   "Topic :: Software Development :: Libraries :: Python Modules",
                   "Programming Language :: Python :: Implementation :: PyPy",
                   "Programming Language :: Python :: Implementation :: CPython",
                   'Programming Language :: Python',
                   "Programming Language :: Python :: 2",
                   "Programming Language :: Python :: 2.7",
                   "Programming Language :: Python :: 3",
                   "Programming Language :: Python :: 3.5",
                   "Programming Language :: Python :: 3.6"
                   ],
      keywords='routes webob dispatch',
      author="Ben Bangert",
      author_email="ben@groovie.org",
      url='https://routes.readthedocs.io/',
      license="MIT",
      test_suite="nose.collector",
      include_package_data=True,
      zip_safe=False,
      tests_require=["soupsieve<2.0", 'nose', 'webtest', 'webob', 'coverage'],
      install_requires=[
          "six",
          "repoze.lru>=0.3"
      ],
      extras_require=extras_require,
      **extra_options
)



import unittest
from routes.route import Route


class TestRouteEscape(unittest.TestCase):
    def test_normal_route(self):
        r = Route('test', '/foo/bar')
        self.assertEqual(r.routelist, ['/foo/bar'])

    def test_route_with_backslash(self):
        r = Route('test', '/foo\\\\bar')
        self.assertEqual(r.routelist, ['/foo\\bar'])

    def test_route_with_random_escapes(self):
        r = Route('test', '\\/f\\oo\\/ba\\r')
        self.assertEqual(r.routelist, ['\\/f\\oo\\/ba\\r'])

    def test_route_with_colon(self):
        r = Route('test', '/foo:bar/baz')
        self.assertEqual(
            r.routelist, ['/foo', {'name': 'bar', 'type': ':'}, '/', 'baz'])

    def test_route_with_escaped_colon(self):
        r = Route('test', '/foo\\:bar/baz')
        self.assertEqual(r.routelist, ['/foo:bar/baz'])

    def test_route_with_both_colons(self):
        r = Route('test', '/prefix/escaped\\:escaped/foo=:notescaped/bar=42')
        self.assertEqual(
            r.routelist, ['/prefix/escaped:escaped/foo=',
                          {'name': 'notescaped', 'type': ':'}, '/', 'bar=42'])

    def test_route_with_all_escapes(self):
        r = Route('test', '/hmm\\:\\*\\{\\}*star/{brackets}/:colon')
        self.assertEqual(
            r.routelist, ['/hmm:*{}', {'name': 'star', 'type': '*'}, '/',
                          {'name': 'brackets', 'type': ':'}, '/',
                          {'name': 'colon', 'type': ':'}])



"""test_generation"""
import sys, time, unittest
from six.moves import urllib

from nose.tools import eq_, assert_raises
from routes import *

class TestGeneration(unittest.TestCase):

    def test_all_static_no_reqs(self):
        m = Mapper()
        m.connect('hello/world')

        eq_('/hello/world', m.generate())

    def test_basic_dynamic(self):
        for path in ['hi/:fred', 'hi/:(fred)']:
            m = Mapper()
            m.connect(path)

            eq_('/hi/index', m.generate(fred='index'))
            eq_('/hi/show', m.generate(fred='show'))
            eq_('/hi/list%20people', m.generate(fred='list people'))
            eq_(None, m.generate())

    def test_relative_url(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.environ = dict(HTTP_HOST='localhost')
        url = URLGenerator(m, m.environ)
        m.connect(':controller/:action/:id')
        m.create_regs(['content','blog','admin/comments'])

        eq_('about', url('about'))
        eq_('http://localhost/about', url('about', qualified=True))

    def test_basic_dynamic_explicit_use(self):
        m = Mapper()
        m.connect('hi/{fred}')
        url = URLGenerator(m, {})

        eq_('/hi/index', url(fred='index'))
        eq_('/hi/show', url(fred='show'))
        eq_('/hi/list%20people', url(fred='list people'))

    def test_dynamic_with_default(self):
        for path in ['hi/:action', 'hi/:(action)']:
            m = Mapper(explicit=False)
            m.minimization = True
            m.connect(path)

            eq_('/hi', m.generate(action='index'))
            eq_('/hi/show', m.generate(action='show'))
            eq_('/hi/list%20people', m.generate(action='list people'))
            eq_('/hi', m.generate())

    def test_dynamic_with_false_equivs(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('article/:page', page=False)
        m.connect(':controller/:action/:id')

        eq_('/blog/view/0', m.generate(controller="blog", action="view", id="0"))
        eq_('/blog/view/0', m.generate(controller="blog", action="view", id=0))
        eq_('/blog/view/False', m.generate(controller="blog", action="view", id=False))
        eq_('/blog/view/False', m.generate(controller="blog", action="view", id='False'))
        eq_('/blog/view', m.generate(controller="blog", action="view", id=None))
        eq_('/blog/view', m.generate(controller="blog", action="view", id='None'))
        eq_('/article', m.generate(page=None))

        m = Mapper()
        m.minimization = True
        m.connect('view/:home/:area', home="austere", area=None)

        eq_('/view/sumatra', m.generate(home='sumatra'))
        eq_('/view/austere/chicago', m.generate(area='chicago'))

        m = Mapper()
        m.minimization = True
        m.connect('view/:home/:area', home=None, area=None)

        eq_('/view/None/chicago', m.generate(home=None, area='chicago'))

    def test_dynamic_with_underscore_parts(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('article/:small_page', small_page=False)
        m.connect(':(controller)/:(action)/:(id)')

        eq_('/blog/view/0', m.generate(controller="blog", action="view", id="0"))
        eq_('/blog/view/False', m.generate(controller="blog", action="view", id='False'))
        eq_('/blog/view', m.generate(controller="blog", action="view", id='None'))
        eq_('/article', m.generate(small_page=None))
        eq_('/article/hobbes', m.generate(small_page='hobbes'))

    def test_dynamic_with_false_equivs_and_splits(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('article/:(page)', page=False)
        m.connect(':(controller)/:(action)/:(id)')

        eq_('/blog/view/0', m.generate(controller="blog", action="view", id="0"))
        eq_('/blog/view/0', m.generate(controller="blog", action="view", id=0))
        eq_('/blog/view/False', m.generate(controller="blog", action="view", id=False))
        eq_('/blog/view/False', m.generate(controller="blog", action="view", id='False'))
        eq_('/blog/view', m.generate(controller="blog", action="view", id=None))
        eq_('/blog/view', m.generate(controller="blog", action="view", id='None'))
        eq_('/article', m.generate(page=None))

        m = Mapper()
        m.minimization = True
        m.connect('view/:(home)/:(area)', home="austere", area=None)

        eq_('/view/sumatra', m.generate(home='sumatra'))
        eq_('/view/austere/chicago', m.generate(area='chicago'))

        m = Mapper()
        m.minimization = True
        m.connect('view/:(home)/:(area)', home=None, area=None)

        eq_('/view/None/chicago', m.generate(home=None, area='chicago'))

    def test_dynamic_with_regexp_condition(self):
        for path in ['hi/:name', 'hi/:(name)']:
            m = Mapper()
            m.connect(path, requirements = {'name':'[a-z]+'})

            eq_('/hi/index', m.generate(name='index'))
            eq_(None, m.generate(name='fox5'))
            eq_(None, m.generate(name='something_is_up'))
            eq_('/hi/abunchofcharacter', m.generate(name='abunchofcharacter'))
            eq_(None, m.generate())

    def test_dynamic_with_default_and_regexp_condition(self):
        for path in ['hi/:action', 'hi/:(action)']:
            m = Mapper(explicit=False)
            m.minimization = True
            m.connect(path, requirements = {'action':'[a-z]+'})

            eq_('/hi', m.generate(action='index'))
            eq_(None, m.generate(action='fox5'))
            eq_(None, m.generate(action='something_is_up'))
            eq_(None, m.generate(action='list people'))
            eq_('/hi/abunchofcharacter', m.generate(action='abunchofcharacter'))
            eq_('/hi', m.generate())

    def test_path(self):
        for path in ['hi/*file', 'hi/*(file)']:
            m = Mapper()
            m.minimization = True
            m.connect(path)

            eq_('/hi', m.generate(file=None))
            eq_('/hi/books/learning_python.pdf', m.generate(file='books/learning_python.pdf'))
            eq_('/hi/books/development%26whatever/learning_python.pdf',
                m.generate(file='books/development&whatever/learning_python.pdf'))

    def test_path_backwards(self):
        for path in ['*file/hi', '*(file)/hi']:
            m = Mapper()
            m.minimization = True
            m.connect(path)

            eq_('/hi', m.generate(file=None))
            eq_('/books/learning_python.pdf/hi', m.generate(file='books/learning_python.pdf'))
            eq_('/books/development%26whatever/learning_python.pdf/hi',
                m.generate(file='books/development&whatever/learning_python.pdf'))

    def test_controller(self):
        for path in ['hi/:controller', 'hi/:(controller)']:
            m = Mapper()
            m.connect(path)

            eq_('/hi/content', m.generate(controller='content'))
            eq_('/hi/admin/user', m.generate(controller='admin/user'))

    def test_controller_with_static(self):
        for path in ['hi/:controller', 'hi/:(controller)']:
            m = Mapper()
            m.connect(path)
            m.connect('google', 'http://www.google.com', _static=True)

            eq_('/hi/content', m.generate(controller='content'))
            eq_('/hi/admin/user', m.generate(controller='admin/user'))
            eq_('http://www.google.com', url_for('google'))

    def test_standard_route(self):
        for path in [':controller/:action/:id', ':(controller)/:(action)/:(id)']:
            m = Mapper(explicit=False)
            m.minimization = True
            m.connect(path)

            eq_('/content', m.generate(controller='content', action='index'))
            eq_('/content/list', m.generate(controller='content', action='list'))
            eq_('/content/show/10', m.generate(controller='content', action='show', id ='10'))

            eq_('/admin/user', m.generate(controller='admin/user', action='index'))
            eq_('/admin/user/list', m.generate(controller='admin/user', action='list'))
            eq_('/admin/user/show/10', m.generate(controller='admin/user', action='show', id='10'))

    def test_multiroute(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('archive/:year/:month/:day', controller='blog', action='view', month=None, day=None,
                            requirements={'month':'\d{1,2}','day':'\d{1,2}'})
        m.connect('viewpost/:id', controller='post', action='view')
        m.connect(':controller/:action/:id')

        url = m.generate(controller='blog', action='view', year=2004, month='blah')
        assert url == '/blog/view?year=2004&month=blah' or url == '/blog/view?month=blah&year=2004'
        eq_('/archive/2004/11', m.generate(controller='blog', action='view', year=2004, month=11))
        eq_('/archive/2004/11', m.generate(controller='blog', action='view', year=2004, month='11'))
        eq_('/archive/2004', m.generate(controller='blog', action='view', year=2004))
        eq_('/viewpost/3', m.generate(controller='post', action='view', id=3))

    def test_multiroute_with_splits(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('archive/:(year)/:(month)/:(day)', controller='blog', action='view', month=None, day=None,
                            requirements={'month':'\d{1,2}','day':'\d{1,2}'})
        m.connect('viewpost/:(id)', controller='post', action='view')
        m.connect(':(controller)/:(action)/:(id)')

        url = m.generate(controller='blog', action='view', year=2004, month='blah')
        assert url == '/blog/view?year=2004&month=blah' or url == '/blog/view?month=blah&year=2004'
        eq_('/archive/2004/11', m.generate(controller='blog', action='view', year=2004, month=11))
        eq_('/archive/2004/11', m.generate(controller='blog', action='view', year=2004, month='11'))
        eq_('/archive/2004', m.generate(controller='blog', action='view', year=2004))
        eq_('/viewpost/3', m.generate(controller='post', action='view', id=3))

    def test_big_multiroute(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('', controller='articles', action='index')
        m.connect('admin', controller='admin/general', action='index')

        m.connect('admin/comments/article/:article_id/:action/:id', controller = 'admin/comments', action=None, id=None)
        m.connect('admin/trackback/article/:article_id/:action/:id', controller='admin/trackback', action=None, id=None)
        m.connect('admin/content/:action/:id', controller='admin/content')

        m.connect('xml/:action/feed.xml', controller='xml')
        m.connect('xml/articlerss/:id/feed.xml', controller='xml', action='articlerss')
        m.connect('index.rdf', controller='xml', action='rss')

        m.connect('articles', controller='articles', action='index')
        m.connect('articles/page/:page', controller='articles', action='index', requirements = {'page':'\d+'})

        m.connect('articles/:year/:month/:day/page/:page', controller='articles', action='find_by_date', month = None, day = None,
                            requirements = {'year':'\d{4}', 'month':'\d{1,2}','day':'\d{1,2}'})
        m.connect('articles/category/:id', controller='articles', action='category')
        m.connect('pages/*name', controller='articles', action='view_page')


        eq_('/pages/the/idiot/has/spoken',
            m.generate(controller='articles', action='view_page', name='the/idiot/has/spoken'))
        eq_('/', m.generate(controller='articles', action='index'))
        eq_('/xml/articlerss/4/feed.xml', m.generate(controller='xml', action='articlerss', id=4))
        eq_('/xml/rss/feed.xml', m.generate(controller='xml', action='rss'))
        eq_('/admin/comments/article/4/view/2',
            m.generate(controller='admin/comments', action='view', article_id=4, id=2))
        eq_('/admin', m.generate(controller='admin/general'))
        eq_('/admin/comments/article/4/index', m.generate(controller='admin/comments', article_id=4))
        eq_('/admin/comments/article/4',
            m.generate(controller='admin/comments', action=None, article_id=4))
        eq_('/articles/2004/2/20/page/1',
            m.generate(controller='articles', action='find_by_date', year=2004, month=2, day=20, page=1))
        eq_('/articles/category', m.generate(controller='articles', action='category'))
        eq_('/xml/index/feed.xml', m.generate(controller='xml'))
        eq_('/xml/articlerss/feed.xml', m.generate(controller='xml', action='articlerss'))

        eq_(None, m.generate(controller='admin/comments', id=2))
        eq_(None, m.generate(controller='articles', action='find_by_date', year=2004))

    def test_big_multiroute_with_splits(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('', controller='articles', action='index')
        m.connect('admin', controller='admin/general', action='index')

        m.connect('admin/comments/article/:(article_id)/:(action)/:(id).html', controller = 'admin/comments', action=None, id=None)
        m.connect('admin/trackback/article/:(article_id)/:action/:(id).html', controller='admin/trackback', action=None, id=None)
        m.connect('admin/content/:(action)/:(id)', controller='admin/content')

        m.connect('xml/:(action)/feed.xml', controller='xml')
        m.connect('xml/articlerss/:(id)/feed.xml', controller='xml', action='articlerss')
        m.connect('index.rdf', controller='xml', action='rss')

        m.connect('articles', controller='articles', action='index')
        m.connect('articles/page/:(page).myt', controller='articles', action='index', requirements = {'page':'\d+'})

        m.connect('articles/:(year)/:month/:(day)/page/:page', controller='articles', action='find_by_date', month = None, day = None,
                            requirements = {'year':'\d{4}', 'month':'\d{1,2}','day':'\d{1,2}'})
        m.connect('articles/category/:id', controller='articles', action='category')
        m.connect('pages/*name', controller='articles', action='view_page')


        eq_('/pages/the/idiot/has/spoken',
            m.generate(controller='articles', action='view_page', name='the/idiot/has/spoken'))
        eq_('/', m.generate(controller='articles', action='index'))
        eq_('/xml/articlerss/4/feed.xml', m.generate(controller='xml', action='articlerss', id=4))
        eq_('/xml/rss/feed.xml', m.generate(controller='xml', action='rss'))
        eq_('/admin/comments/article/4/view/2.html',
            m.generate(controller='admin/comments', action='view', article_id=4, id=2))
        eq_('/admin', m.generate(controller='admin/general'))
        eq_('/admin/comments/article/4/edit/3.html',
            m.generate(controller='admin/comments', article_id=4, action='edit', id=3))
        eq_(None, m.generate(controller='admin/comments', action=None, article_id=4))
        eq_('/articles/2004/2/20/page/1',
            m.generate(controller='articles', action='find_by_date', year=2004, month=2, day=20, page=1))
        eq_('/articles/category', m.generate(controller='articles', action='category'))
        eq_('/xml/index/feed.xml', m.generate(controller='xml'))
        eq_('/xml/articlerss/feed.xml', m.generate(controller='xml', action='articlerss'))

        eq_(None, m.generate(controller='admin/comments', id=2))
        eq_(None, m.generate(controller='articles', action='find_by_date', year=2004))

    def test_big_multiroute_with_nomin(self):
        m = Mapper(explicit=False)
        m.minimization = False
        m.connect('', controller='articles', action='index')
        m.connect('admin', controller='admin/general', action='index')

        m.connect('admin/comments/article/:article_id/:action/:id', controller = 'admin/comments', action=None, id=None)
        m.connect('admin/trackback/article/:article_id/:action/:id', controller='admin/trackback', action=None, id=None)
        m.connect('admin/content/:action/:id', controller='admin/content')

        m.connect('xml/:action/feed.xml', controller='xml')
        m.connect('xml/articlerss/:id/feed.xml', controller='xml', action='articlerss')
        m.connect('index.rdf', controller='xml', action='rss')

        m.connect('articles', controller='articles', action='index')
        m.connect('articles/page/:page', controller='articles', action='index', requirements = {'page':'\d+'})

        m.connect('articles/:year/:month/:day/page/:page', controller='articles', action='find_by_date', month = None, day = None,
                            requirements = {'year':'\d{4}', 'month':'\d{1,2}','day':'\d{1,2}'})
        m.connect('articles/category/:id', controller='articles', action='category')
        m.connect('pages/*name', controller='articles', action='view_page')


        eq_('/pages/the/idiot/has/spoken',
            m.generate(controller='articles', action='view_page', name='the/idiot/has/spoken'))
        eq_('/', m.generate(controller='articles', action='index'))
        eq_('/xml/articlerss/4/feed.xml', m.generate(controller='xml', action='articlerss', id=4))
        eq_('/xml/rss/feed.xml', m.generate(controller='xml', action='rss'))
        eq_('/admin/comments/article/4/view/2',
            m.generate(controller='admin/comments', action='view', article_id=4, id=2))
        eq_('/admin', m.generate(controller='admin/general'))
        eq_('/articles/2004/2/20/page/1',
            m.generate(controller='articles', action='find_by_date', year=2004, month=2, day=20, page=1))
        eq_(None, m.generate(controller='articles', action='category'))
        eq_('/articles/category/4', m.generate(controller='articles', action='category', id=4))
        eq_('/xml/index/feed.xml', m.generate(controller='xml'))
        eq_('/xml/articlerss/feed.xml', m.generate(controller='xml', action='articlerss'))

        eq_(None, m.generate(controller='admin/comments', id=2))
        eq_(None, m.generate(controller='articles', action='find_by_date', year=2004))

    def test_no_extras(self):
        m = Mapper()
        m.minimization = True
        m.connect(':controller/:action/:id')
        m.connect('archive/:year/:month/:day', controller='blog', action='view', month=None, day=None)

        eq_('/archive/2004', m.generate(controller='blog', action='view', year=2004))

    def test_no_extras_with_splits(self):
        m = Mapper()
        m.minimization = True
        m.connect(':(controller)/:(action)/:(id)')
        m.connect('archive/:(year)/:(month)/:(day)', controller='blog', action='view', month=None, day=None)

        eq_('/archive/2004', m.generate(controller='blog', action='view', year=2004))

    def test_the_smallest_route(self):
        for path in ['pages/:title', 'pages/:(title)']:
            m = Mapper()
            m.connect('', controller='page', action='view', title='HomePage')
            m.connect(path, controller='page', action='view')

            eq_('/', m.generate(controller='page', action='view', title='HomePage'))
            eq_('/pages/joe', m.generate(controller='page', action='view', title='joe'))

    def test_extras(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('viewpost/:id', controller='post', action='view')
        m.connect(':controller/:action/:id')

        eq_('/viewpost/2?extra=x%2Fy', m.generate(controller='post', action='view', id=2, extra='x/y'))
        eq_('/blog?extra=3', m.generate(controller='blog', action='index', extra=3))
        eq_('/viewpost/2?extra=3', m.generate(controller='post', action='view', id=2, extra=3))

    def test_extras_with_splits(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('viewpost/:(id)', controller='post', action='view')
        m.connect(':(controller)/:(action)/:(id)')

        eq_('/blog?extra=3', m.generate(controller='blog', action='index', extra=3))
        eq_('/viewpost/2?extra=3', m.generate(controller='post', action='view', id=2, extra=3))

    def test_extras_as_unicode(self):
        m = Mapper()
        m.connect(':something')
        thing = "whatever"
        euro = u"\u20ac" # Euro symbol

        eq_("/%s?extra=%%E2%%82%%AC" % thing, m.generate(something=thing, extra=euro))

    def test_extras_as_list_of_unicodes(self):
        m = Mapper()
        m.connect(':something')
        thing = "whatever"
        euro = [u"\u20ac", u"\xa3"] # Euro and Pound sterling symbols

        eq_("/%s?extra=%%E2%%82%%AC&extra=%%C2%%A3" % thing, m.generate(something=thing, extra=euro))


    def test_static(self):
        m = Mapper()
        m.connect('hello/world',known='known_value',controller='content',action='index')

        eq_('/hello/world', m.generate(controller='content',action= 'index',known ='known_value'))
        eq_('/hello/world?extra=hi',
            m.generate(controller='content',action='index',known='known_value',extra='hi'))

        eq_(None, m.generate(known='foo'))

    def test_typical(self):
        for path in [':controller/:action/:id', ':(controller)/:(action)/:(id)']:
            m = Mapper()
            m.minimization = True
            m.minimization = True
            m.connect(path, action = 'index', id = None)

            eq_('/content', m.generate(controller='content', action='index'))
            eq_('/content/list', m.generate(controller='content', action='list'))
            eq_('/content/show/10', m.generate(controller='content', action='show', id=10))

            eq_('/admin/user', m.generate(controller='admin/user', action='index'))
            eq_('/admin/user', m.generate(controller='admin/user'))
            eq_('/admin/user/show/10', m.generate(controller='admin/user', action='show', id=10))

            eq_('/content', m.generate(controller='content'))

    def test_route_with_fixnum_default(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('page/:id', controller='content', action='show_page', id=1)
        m.connect(':controller/:action/:id')

        eq_('/page', m.generate(controller='content', action='show_page'))
        eq_('/page', m.generate(controller='content', action='show_page', id=1))
        eq_('/page', m.generate(controller='content', action='show_page', id='1'))
        eq_('/page/10', m.generate(controller='content', action='show_page', id=10))

        eq_('/blog/show/4', m.generate(controller='blog', action='show', id=4))
        eq_('/page', m.generate(controller='content', action='show_page'))
        eq_('/page/4', m.generate(controller='content', action='show_page',id=4))
        eq_('/content/show', m.generate(controller='content', action='show'))

    def test_route_with_fixnum_default_with_splits(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('page/:(id)', controller='content', action='show_page', id =1)
        m.connect(':(controller)/:(action)/:(id)')

        eq_('/page', m.generate(controller='content', action='show_page'))
        eq_('/page', m.generate(controller='content', action='show_page', id=1))
        eq_('/page', m.generate(controller='content', action='show_page', id='1'))
        eq_('/page/10', m.generate(controller='content', action='show_page', id=10))

        eq_('/blog/show/4', m.generate(controller='blog', action='show', id=4))
        eq_('/page', m.generate(controller='content', action='show_page'))
        eq_('/page/4', m.generate(controller='content', action='show_page',id=4))
        eq_('/content/show', m.generate(controller='content', action='show'))

    def test_uppercase_recognition(self):
        for path in [':controller/:action/:id', ':(controller)/:(action)/:(id)']:
            m = Mapper(explicit=False)
            m.minimization = True
            m.connect(path)

            eq_('/Content', m.generate(controller='Content', action='index'))
            eq_('/Content/list', m.generate(controller='Content', action='list'))
            eq_('/Content/show/10', m.generate(controller='Content', action='show', id='10'))

            eq_('/Admin/NewsFeed', m.generate(controller='Admin/NewsFeed', action='index'))

    def test_backwards(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('page/:id/:action', controller='pages', action='show')
        m.connect(':controller/:action/:id')

        eq_('/page/20', m.generate(controller='pages', action='show', id=20))
        eq_('/pages/boo', m.generate(controller='pages', action='boo'))

    def test_backwards_with_splits(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect('page/:(id)/:(action)', controller='pages', action='show')
        m.connect(':(controller)/:(action)/:(id)')

        eq_('/page/20', m.generate(controller='pages', action='show', id=20))
        eq_('/pages/boo', m.generate(controller='pages', action='boo'))

    def test_both_requirement_and_optional(self):
        m = Mapper()
        m.minimization = True
        m.connect('test/:year', controller='post', action='show', year=None, requirements = {'year':'\d{4}'})

        eq_('/test', m.generate(controller='post', action='show'))
        eq_('/test', m.generate(controller='post', action='show', year=None))

    def test_set_to_nil_forgets(self):
        m = Mapper()
        m.minimization = True
        m.connect('pages/:year/:month/:day', controller='content', action='list_pages', month=None, day=None)
        m.connect(':controller/:action/:id')

        eq_('/pages/2005', m.generate(controller='content', action='list_pages', year=2005))
        eq_('/pages/2005/6', m.generate(controller='content', action='list_pages', year=2005, month=6))
        eq_('/pages/2005/6/12',
            m.generate(controller='content', action='list_pages', year=2005, month=6, day=12))

    def test_url_with_no_action_specified(self):
        m = Mapper()
        m.connect('', controller='content')
        m.connect(':controller/:action/:id')

        eq_('/', m.generate(controller='content', action='index'))
        eq_('/', m.generate(controller='content'))

    def test_url_with_prefix(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.prefix = '/blog'
        m.connect(':controller/:action/:id')
        m.create_regs(['content','blog','admin/comments'])

        eq_('/blog/content/view', m.generate(controller='content', action='view'))
        eq_('/blog/content', m.generate(controller='content'))
        eq_('/blog/admin/comments', m.generate(controller='admin/comments'))

    def test_url_with_prefix_deeper(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.prefix = '/blog/phil'
        m.connect(':controller/:action/:id')
        m.create_regs(['content','blog','admin/comments'])

        eq_('/blog/phil/content/view', m.generate(controller='content', action='view'))
        eq_('/blog/phil/content', m.generate(controller='content'))
        eq_('/blog/phil/admin/comments', m.generate(controller='admin/comments'))

    def test_url_with_environ_empty(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.environ = dict(SCRIPT_NAME='')
        m.connect(':controller/:action/:id')
        m.create_regs(['content','blog','admin/comments'])

        eq_('/content/view', m.generate(controller='content', action='view'))
        eq_('/content', m.generate(controller='content'))
        eq_('/admin/comments', m.generate(controller='admin/comments'))

    def test_url_with_environ(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.environ = dict(SCRIPT_NAME='/blog')
        m.connect(':controller/:action/:id')
        m.create_regs(['content','blog','admin/comments'])

        eq_('/blog/content/view', m.generate(controller='content', action='view'))
        eq_('/blog/content', m.generate(controller='content'))
        eq_('/blog/content', m.generate(controller='content'))
        eq_('/blog/admin/comments', m.generate(controller='admin/comments'))

        m.environ = dict(SCRIPT_NAME='/notblog')

        eq_('/notblog/content/view', m.generate(controller='content', action='view'))
        eq_('/notblog/content', m.generate(controller='content'))
        eq_('/notblog/content', m.generate(controller='content'))
        eq_('/notblog/admin/comments', m.generate(controller='admin/comments'))

    def test_url_with_environ_and_caching(self):
        m = Mapper()
        m.connect("foo", "/", controller="main", action="index")

        eq_('/', m.generate(controller='main', action='index'))
        eq_('/bar/', m.generate(controller='main', action='index', _environ=dict(SCRIPT_NAME='/bar')))
        eq_('/', m.generate(controller='main', action='index'))

    def test_url_with_environ_and_absolute(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.environ = dict(SCRIPT_NAME='/blog')
        m.connect('image', 'image/:name', _absolute=True)
        m.connect(':controller/:action/:id')
        m.create_regs(['content','blog','admin/comments'])

        eq_('/blog/content/view', m.generate(controller='content', action='view'))
        eq_('/blog/content', m.generate(controller='content'))
        eq_('/blog/content', m.generate(controller='content'))
        eq_('/blog/admin/comments', m.generate(controller='admin/comments'))
        eq_('/image/topnav.jpg', url_for('image', name='topnav.jpg'))

    def test_route_with_odd_leftovers(self):
        m = Mapper(explicit=False)
        m.minimization = True
        m.connect(':controller/:(action)-:(id)')
        m.create_regs(['content','blog','admin/comments'])

        eq_('/content/view-', m.generate(controller='content', action='view'))
        eq_('/content/index-', m.generate(controller='content'))

    def test_route_with_end_extension(self):
        m = Mapper(explicit=False)
        m.connect(':controller/:(action)-:(id).html')
        m.create_regs(['content','blog','admin/comments'])

        eq_(None, m.generate(controller='content', action='view'))
        eq_(None, m.generate(controller='content'))

        eq_('/content/view-3.html', m.generate(controller='content', action='view', id=3))
        eq_('/content/index-2.html', m.generate(controller='content', id=2))

    def test_unicode(self):
        hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
        hoge_enc = urllib.parse.quote(hoge.encode('utf-8'))
        m = Mapper()
        m.connect(':hoge')
        eq_("/%s" % hoge_enc, m.generate(hoge=hoge))
        self.assert_(isinstance(m.generate(hoge=hoge), str))

    def test_unicode_static(self):
        hoge = u'\u30c6\u30b9\u30c8' # the word test in Japanese
        hoge_enc = urllib.parse.quote(hoge.encode('utf-8'))
        m = Mapper()
        m.minimization = True
        m.connect('google-jp', 'http://www.google.co.jp/search', _static=True)
        m.create_regs(['messages'])
        eq_("http://www.google.co.jp/search?q=" + hoge_enc, url_for('google-jp', q=hoge))
        self.assert_(isinstance(url_for('google-jp', q=hoge), str))

    def test_other_special_chars(self):
        m = Mapper()
        m.minimization = True
        m.connect('/:year/:(slug).:(format),:(locale)', locale='en', format='html')
        m.create_regs(['content'])

        eq_('/2007/test', m.generate(year=2007, slug='test'))
        eq_('/2007/test.xml', m.generate(year=2007, slug='test', format='xml'))
        eq_('/2007/test.xml,ja', m.generate(year=2007, slug='test', format='xml', locale='ja'))
        eq_(None, m.generate(year=2007, format='html'))

    def test_dot_format_args(self):
        for minimization in [False, True]:
            m = Mapper(explicit=True)
            m.minimization=minimization
            m.connect('/songs/{title}{.format}')
            m.connect('/stories/{slug}{.format:pdf}')

            eq_('/songs/my-way', m.generate(title='my-way'))
            eq_('/songs/my-way.mp3', m.generate(title='my-way', format='mp3'))
            eq_('/stories/frist-post', m.generate(slug='frist-post'))
            eq_('/stories/frist-post.pdf', m.generate(slug='frist-post', format='pdf'))
            eq_(None, m.generate(slug='frist-post', format='doc'))

if __name__ == '__main__':
    unittest.main()
else:
    def bench_gen(withcache = False):
        m = Mapper()
        m.connect('', controller='articles', action='index')
        m.connect('admin', controller='admin/general', action='index')

        m.connect('admin/comments/article/:article_id/:action/:id', controller = 'admin/comments', action = None, id=None)
        m.connect('admin/trackback/article/:article_id/:action/:id', controller='admin/trackback', action=None, id=None)
        m.connect('admin/content/:action/:id', controller='admin/content')

        m.connect('xml/:action/feed.xml', controller='xml')
        m.connect('xml/articlerss/:id/feed.xml', controller='xml', action='articlerss')
        m.connect('index.rdf', controller='xml', action='rss')

        m.connect('articles', controller='articles', action='index')
        m.connect('articles/page/:page', controller='articles', action='index', requirements = {'page':'\d+'})

        m.connect('articles/:year/:month/:day/page/:page', controller='articles', action='find_by_date', month = None, day = None,
                            requirements = {'year':'\d{4}', 'month':'\d{1,2}','day':'\d{1,2}'})
        m.connect('articles/category/:id', controller='articles', action='category')
        m.connect('pages/*name', controller='articles', action='view_page')
        if withcache:
            m.urlcache = {}
        m._create_gens()
        n = 5000
        start = time.time()
        for x in range(1,n):
            m.generate(controller='articles', action='index', page=4)
            m.generate(controller='admin/general', action='index')
            m.generate(controller='admin/comments', action='show', article_id=2)

            m.generate(controller='articles', action='find_by_date', year=2004, page=1)
            m.generate(controller='articles', action='category', id=4)
            m.generate(controller='xml', action='articlerss', id=2)
        end = time.time()
        ts = time.time()
        for x in range(1,n*6):
            pass
        en = time.time()
        total = end-start-(en-ts)
        per_url = total / (n*6)
        print("Generation (%s URLs)" % (n*6))
        print("%s ms/url" % (per_url*1000))
        print("%s urls/s\n" % (1.00/per_url))
